import sys
import asyncio
from pathlib import Path
from importlib import import_module

from aiohttp import web
from ajgu import Storage
from ajgu import GraphDatabase

from .render import render as _render


if __name__ == '__main__':
    if len(sys.argv) < 2:
        print('Usage:')
        print('  python -m ajgu.http.server PYTHON.MODULE.PATH:STORAGE_CLASS')
        print('\n e.g. python -m ajgu.http.server ajgu.storage.bsddb3:BSDDB3')
        sys.exit(1)

    module_path, storage_name = sys.argv[1].split(':')
    storage_class = getattr(import_module(module_path), storage_name)

    DATABASE_PATH = '.'


ROOT = (Path(__file__) / '..').resolve()
print("database @", ROOT)
print("storage class @", storage_class)

render = lambda template, **context: _render(template, str(ROOT), **context)


storage = storage_class(
    DATABASE_PATH,
    (0, 1024**2),
    (0, 2*1024**2)
)

db = GraphDatabase(storage)


@asyncio.coroutine
def index(request):
    print('index')
    page = int(request.GET.get('page', '1'))
    COUNT = 40
    start = (page - 1) * COUNT
    end = start + COUNT
    with db.transaction() as txn:
        vertices = txn.vertex.slice(start, end)
        text = render('index.jinja', vertices=vertices, next=page+1)
    return web.Response(body=text.encode('utf-8'))


@asyncio.coroutine
def vertex(request):
    print('vertex')
    identifier = request.match_info.get('identifier')
    with db.transaction() as txn:
        vertex = txn.vertex.get(identifier)
        text = render('vertex.jinja', vertex=vertex)
    return web.Response(body=text.encode('utf-8'))


@asyncio.coroutine
def edge(request):
    print('edge')
    app = web.Application(loop=loop)
    identifier = request.match_info.get('identifier')
    with db.transaction() as txn:
        print('echo')
        edge = txn.edge.get(identifier)
        text = render('edge.jinja', edge=edge)
    return web.Response(body=text.encode('utf-8'))


@asyncio.coroutine
def init(loop):
    app = web.Application(loop=loop)
    app.router.add_static('/static', str(ROOT / 'static'))
    app.router.add_route('GET', '/', index)
    app.router.add_route('GET', '/vertex/{identifier}', vertex)
    app.router.add_route('GET', '/edge/{identifier}', edge)
    srv = yield from loop.create_server(
        app.make_handler(),
        '0.0.0.0',
        8080
    )
    print("Server started at http://0.0.0.0:8080")
    return srv


if __name__ == '__main__':
    loop = asyncio.get_event_loop()
    loop.run_until_complete(init(loop))
    try:
        loop.run_forever()
    except KeyboardInterrupt:
        db.close()
