import sys
from datetime import datetime
from ajgu.storage.bsddb3 import NoTxnBSDDB3
from ajgu import GraphDatabase

import msgpack

progress = 'checkpoint {} {:f}%'


def main():
    storage = NoTxnBSDDB3('./_db', (3, 0), (4, 0))
    graph = GraphDatabase(storage)

    keys = [
        'sources',
        'surfaceText',
        'source_uri',
        'dataset',
        'id',
        'uri',
        'features',
        'context',
        'weight',
        'license'
    ]
    STEP = datetime.now()
    try:
        print('Starting load in `_db/`')
        print('This takes several hours on an average hardware!')
        for index in range(8):
            name = './part_0{}.msgpack'.format(index)
            with open(name, 'rb') as stream:
                unpacker = msgpack.Unpacker(stream, encoding='utf-8')
                for num, value in enumerate(unpacker):
                    with graph.transaction(sync=False) as txn:
                        start = list(txn.vertex.label(value['start']).all())
                        if start:
                            start = start[0]
                        else:
                            start = txn.vertex.create(value['start'])
                        end = list(txn.vertex.label(value['end']).all())
                        if end:
                            end = end[0]
                        else:
                            end = txn.vertex.create(value['end'])
                        relation = txn.edge.create(start, value['rel'], end)
                        for key in keys:
                            relation.props[key] = value[key]
                        if num % 100000 == 0:
                            print(num, datetime.now() - STEP)
                            STEP = datetime.now()
            print(progress.format(name, num))
    except Exception as exc:
        print(exc)
    print('closing')
    graph.close()
    sys.exit()


if __name__ == '__main__':
    main()
