import sys
from csv import reader

from ajgu.storage.bsddb3 import NoTxnBSDDB3 as Storage
from ajgu.storage.file import FileStorage as Storage
from ajgu import GraphDatabase


def main():
    storage = Storage('./db')
    graph = GraphDatabase(storage)

    # create indices
    graph.index('userid')
    graph.index('movieid')    
    graph.index('tag')

    print('Starting load of `movies.csv` in `db/`')
    with open('./ml-latest-small/movies.csv', 'r') as f:
        next(reader(f))  # skip heading
        with graph.transaction() as txn:
            for id, title, genres in reader(f):
                movie = txn.vertex.create('movie')
                movie.props['movieid'] = id
                movie.props['title'] = title
                movie.props['genres'] = genres.split('|')

    print('Starting load of `ratings.csv` in `db/`')
    with open('./ml-latest-small/ratings.csv', 'r') as f:
        next(reader(f))  # skip heading
        for userid, movieid, value, timestamp in reader(f):
            with graph.transaction() as txn:
                # get or create user
                user = txn.vertex.filter(userid=userid).first()
                if not user:
                    user = txn.vertex.create('user')
                    user.props['userid'] = userid
                # create edge between movie and user
                movie = txn.vertex.filter(movieid=movieid).first()
                rating = txn.edge.create(user, 'rating', movie)

                rating.props['value'] = float(value)
                rating.props['timestamp'] = float(timestamp)

    print('Starting load of `tags.csv` in `db/`')
    with open('./ml-latest-small/tags.csv', 'r') as f:
        next(reader(f))  # skip heading
        with graph.transaction() as txn:
            for userid, movieid, value, timestamp in reader(f):
                movie = txn.vertex.filter(movieid=movieid).first()
                user = txn.vertex.filter(userid=userid).first()
                tag = txn.edge.create(user, 'tag', movie)
                tag.props['value'] = value
                tag.props['timestamp'] = float(timestamp)

    print('closing')
    graph.close()
    sys.exit()


if __name__ == '__main__':
    main()
